import time
from soulbot import SoulBot

class Motor:   
    def __init__(self, iss, oss, op, c_op):
        self.__iss = iss
        self.__oss = oss
        self.__motorOp = op
        self.__ctrlOp = c_op

        self.status = [0,0]

        self.__nReq = 0
        self.__nAck = 0

    def parseMotorArgs(self, lwheel, rwheel):
        lw = int(round(128*lwheel))
        rw = int(round(128*rwheel))

        if lw > 127: lw = 127
        elif lw < 0:
            lw += 255
            if lw < 128: lw = 128

        if rw > 127: rw = 127
        elif rw < 0:
            rw += 255
            if rw < 128: rw = 128

        return self.__motorOp.genOp(lw,rw)
    
    
    def move(self, lwheel, rwheel, t=-1):
        cmd = self.parseMotorArgs(lwheel, -rwheel)
        self.__oss(cmd)
        self.__nReq += 1          #Request Send count
        r = SoulBot.checkResponse(self.__motorOp, self.__iss)
        self.__nAck += 1          #Ack Recv count
        if not r: return False
        elif t < 0:
            self.status = [lwheel, rwheel]
            return True
        else:
            w = tuple(self.status)
            time.sleep(t)
            while not self.move(w[0], w[1]): continue 
            return True

    def forward(self, vel=1, t=-1):
        return self.move(vel, vel, t)

    def backward(self, vel=1, t=-1):
        return self.move(-vel, -vel, t)

    def rotate(self, vel=1, t=-1):
        return self.move(vel, -vel, t)

    def pvl(self, vel=1, t=-1):
        return self.move(0, vel, t)

    def pvr(self, vel=1, t=-1):
        return self.move(vel, 0, t)

    def stop(self, t=-1):
        return self.move(0, 0, t)

    def nReq(self):
        return self.__nReq

    def nAck(self):
        return self.__nAck

    def saveCalibration(self):
        self.status = [0,0]
        self.__oss( self.__ctrlOp.genOp() )

        return SoulBot.checkResponse(self.__ctrlOp, self.__iss)

    def clear(self):
        self.status[0] = 0
        self.status[1] = 0

                
    

