# -*- coding: cp1252 -*-

import sys
import op
import time
from uabotexception import *
try:
    from serial import *
except:
    sys.stderr.write("pyserial module is not accessible")


class DataBot:
    def __init__(self, name):
        self.name = name
        self.bSnd = 0
        self.bRcv = 0
        

class SoulBot(DataBot):
    SOP = op.SetOpBot()

    def __init__(self, name="UABOT"):
        DataBot.__init__(self, name)
        self.__conn = False

    def writeRawData(self, l):
        try:
            self.__conn.write(l)
            self.bSnd += len(l)
        except SerialTimeoutException:
            self.__conn.flushOutput()
            raise UABOTExceptionTimeout("Timeout")
        except:
            raise UABOTException(0, "Something goes wrong. Check serial connection")

    def readRawData(self, size=1):
        try:
            cmd = self.__conn.read(size)
            self.bRcv += len(cmd)
            return cmd
        except:
            raise UABOTExceptionConnect("Wrong serial comunication")

    def connect(self, dev, br=38400, ti=1, itc=0.5):
        self.close()
            
        try:
            self.__conn = Serial(dev, br, timeout=ti, interCharTimeout=itc)
            self.__conn.open()
        except (SerialException, ValueError):
            raise UABOTExceptionConnect("Busy port or bad parameter. Check serial params")

    def isConnect(self):
        try:
            return self.__conn.isOpen()
        except:
            return False

    def flushOutput(self):
        self.__conn.flushOutput()
        return True

    def flushInput(self):
        self.__conn.flushInput()
        return True

    def flushAll(self):
        self.__conn.flushInput()
        self.__conn.flushOutput()
        return True
        
    def close(self):
        try:
            self.flushAll()
            self.__conn.close()
        except:
            pass
        return True

    def __del__(self):
        self.close()

    @staticmethod
    def checkResponse(op, iss):
        ack = iss(2)
        if not ack: raise UABOTExceptionTimeout("Timeout, no response. Check connection")
        ack = ( ord(ack[0]), ord(ack[1]) )
            
        if ack[0] == op[0]:        
            if ack[1] == 0: return True             
            else:
                s = iss(ack[1])
                if len(s) == ack[1]: return s
                else: raise UABOTExceptionFrame("ACK Frame has not required lenght") 
        elif ack[0] == 0 and ack[1] == op[0]:
            return False
        else:
            raise UABOTExceptionFrame("Bad frame received <Opcode expected: 0x%02x, Opcode received: 0x%02x>" % (op[0], ord(ack[0])))
    
   

            
        
