VERSION = "0.9"
AUTHOR = "Josep Ma Sanz Xicola"

from soulbot import *
from uabotexception import *
from motor import Motor
from sensor import Sensor
from misc import Misc


def info(object, spacing = 10, collapse = 1):
    """Print methods and doc string.

    Take module, class, list, dictionary, or strings.
    
    Method taken from Dive into Python book (Chapter 4, pag. 32) 
    Copyright Mark Pilgrim"""
    
    methodList = [method for method in dir(object) if callable(getattr(object, method))]
    processFunc = collapse and (lambda s: " ".join(s.split())) or (lambda s: s)
    print "\n".join(["%s %s" % 
                     (method.ljust(spacing), 
                     processFunc(str(getattr(object, method).__doc__)))
                    for method in methodList])
                    

class Uabot(SoulBot):
    """
    """    
    def __init__(self, name="UABOT"):
        SoulBot.__init__(self, name)
        self.motor = Motor( self.readRawData, 
                            self.writeRawData, 
                            Uabot.SOP["MOT"],
                            Uabot.SOP["WCM"])
        
        self.sensor = Sensor(self.readRawData,
                             self.writeRawData,
                             Uabot.SOP.group("SENSOR") )
        
        self.misc = Misc(   self.readRawData,
                            self.writeRawData,
                            Uabot.SOP.group("MISC") )

    def connect(self, dev, br=38400, ti=2, itc=1 ):
        SoulBot.connect(self, dev, br, ti, itc)

        try:
            self.reset()
        except:
            self.close()
            sys.stderr.write("Not received ping task response. Try again\n")
            print sys.exc_info()[0]
            
            

    def nReq(self):
        return self.motor.nReq() + self.sensor.nReq() + self.misc.nReq()

    def nAck(self):
        return self.motor.nAck() + self.sensor.nAck() + self.misc.nAck()


    def reset(self):
        try:
            op = Uabot.SOP["RST"]
            self.writeRawData(op.genOp())

            self.testTaskMotor()
            self.testTaskSensor()

            self.motor.clear()
            self.sensor.clear()
            self.misc.clear()
        except:
            sys.stderr.write("Something goes wrong. Try again\n")
        

    def testTaskMotor(self):
        op = Uabot.SOP["TSTM"]
        self.writeRawData(op.genOp())
        msg = Uabot.checkResponse(op, self.readRawData)
        if msg: print msg.replace("\x00", "").strip()
        else: sys.stderr.write("M-Task is not ready\n")
        

    def testTaskSensor(self):
        op = Uabot.SOP["TSTS"]
        self.writeRawData(op.genOp())
        msg = Uabot.checkResponse(op, self.readRawData)
        if msg: print msg.replace("\x00", "").strip()
        else: sys.stderr.write("S-Task is not ready\n")
    

    @staticmethod
    def ld(value, pos):
        if value & (0x1 << pos): return True
        else: return False



            
